#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2021 T.Nakahama.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#



from gnuradio import gr
#import package for using usrp 
from gnuradio import uhd
import time
from datetime import datetime as dt

class USRP_UTC_Sink(gr.hier_block2):
    """
    docstring for block USRP_UTC_Sink
    """
    def __init__(self, device_address, device_arguments, subdev_spec,num_channels, samp_rate,lo_mode,rf_freq,dsp_freq,antenna0,antenna1,gain_value0,gain_value1,start_time,fractional_time):
        gr.hier_block2.__init__(self,
            "USRP_UTC_Sink",
            gr.io_signature(num_channels, num_channels, gr.sizeof_gr_complex),  # Input signature
            gr.io_signature(0,0,0)) # Output signature

        # Define blocks and connect them
        #Define USRP Block
        self.uhd_usrp_sink_0 = uhd.usrp_sink(
            ",".join((device_address, device_arguments)),
            uhd.stream_args(
                cpu_format="fc32",
                args='',
                channels=list(range(0,num_channels)),
            ),
            '',
        )
        if subdev_spec != "":
            self.uhd_usrp_sink_0.set_subdev_spec(subdev_spec, 0)
        #Reference external clock and PPS
        self.uhd_usrp_sink_0.set_time_source('external', 0)
        self.uhd_usrp_sink_0.set_clock_source('external', 0)
        self.uhd_usrp_sink_0.set_gain(gain_value0, 0)
        self.uhd_usrp_sink_0.set_antenna(antenna0, 0)
        if num_channels == 2:
            self.uhd_usrp_sink_0.set_gain(gain_value1, 1)
            self.uhd_usrp_sink_0.set_antenna(antenna1, 1)
        self.uhd_usrp_sink_0.set_samp_rate(samp_rate)
        #Synchronization setting
        #Set UTC to usrp with PPS
        last_time = self.uhd_usrp_sink_0.get_time_last_pps()
        next_time = self.uhd_usrp_sink_0.get_time_last_pps()
        while(last_time==next_time):
            time.sleep(0.01)
            last_time = next_time
            next_time = self.uhd_usrp_sink_0.get_time_last_pps()
        self.uhd_usrp_sink_0.set_time_next_pps(uhd.time_spec(int(time.time())+1))
        time.sleep(1)
        #Frequency tuuning
        tune_req = uhd.tune_request(dsp_freq=dsp_freq, rf_freq=rf_freq)
        tune_req.dsp_freq_policy = uhd.tune_request.POLICY_MANUAL
        tune_req.rf_freq_policy = uhd.tune_request.POLICY_MANUAL
        tune_req.args = uhd.device_addr("mode_n="+lo_mode)
        #LO offset
        cmd_time = self.uhd_usrp_sink_0.get_time_last_pps() + uhd.time_spec(1.5)
        actual_tune = []
        self.uhd_usrp_sink_0.set_command_time(cmd_time)
        for i in range(num_channels):
            actual_tune.append(self.uhd_usrp_sink_0.set_center_freq(tune_req, i))
        self.uhd_usrp_sink_0.clear_command_time()
        if rf_freq != 0:
            while self.uhd_usrp_sink_0.get_sensor("lo_locked").to_bool() != True:
                pass
        print("===TX Frequency Info===")
        for i in range(num_channels):
            print("ch:"+str(i))
            print("  LO Freq:" + str(actual_tune[i].actual_rf_freq))
            print("  DSP Freq:" + str(actual_tune[i].actual_dsp_freq))
        #Set start timing
        start_time = dt.strptime(start_time,'%Y-%m-%d %H:%M:%S')
        self.uhd_usrp_sink_0.set_start_time(uhd.time_spec(start_time.timestamp())+uhd.time_spec(fractional_time))

        #Define connection
        for i in range(num_channels):
            self.connect((self,i),(self.uhd_usrp_sink_0,i))
